#include <netdb.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <unistd.h>

#include <sys/socket.h>
#include "log.h"


void    ExtractIPNUM(char *linein, struct ip_array_s *ipa)
{
    int             k, l, n;
    int             ip_valid;
    unsigned char   ipnum[4], c;


    /* Perform a down-and-dirty check for a pre-resolved IP     *
     * address.  If a non-digit appears as the first chraracter,*
     * the entry is considered to already have been resolved.   */
    ipa->iplong = 0;
    c = linein[0];
    if((c < '0') || (c > '9'))
    {
        statistics.pre_resolved ++;
        return;
    }

    /* Parse the IP address.  The IP address is value checked   *
     * against many bogus conditions.  All octets must be three *
     * characters or less.  Non-numeric characters are flagged  *
     * and rejected.  More than four octets appearing are       *
     * flagged and rejected.                                    */
    ip_valid = 1;
    for (k = 0, l = 0, n = 0; (k < 4) && ip_valid; k++, l = ++n) 
    {
        c = linein[n];
        while((c >= '0') && (c <= '9'))
        {
            n ++;
            c = linein[n];
        }
        if(((c != '.') && (c != ' ')) || ((c == ' ') && (k != 3)) || ((n-l) > 3))
        {
            ip_valid = 0;
            continue;
        }
        linein[n] = '\0';
        ipnum[k] = atoi(&linein[l]);
        linein[n] = c;
    }

    /* If the IP address is not valid, we should consider it as *
     * having already been resolved.                            */
    if(!ip_valid)
    {
        statistics.fake_ip_addr ++;
        return;
    }

    /* Regardless of whether a reverse DNS lookup will be       *
     * sucessful, the output will now appear as two separate    *
     * entries.  The first entry is the domain name.  If the    *
     * reverse DNS lookup fails, then the domain name will      *
     * point to a text representation of the IP address.  If    *
     * reverse DNS lookup is successful, then the correct       *
     * domain name is pointed to.                               */
    ipa->lineout = &linein[n];
    ipa->iplong = (ipnum[0] << 24) + (ipnum[1] << 16) +
                  (ipnum[2] << 8) + ipnum[3];

}


unsigned long
CountUniqueIPNUMs(unsigned long NumEntries)
{
    unsigned long   sortindex, ip = -1, i;
    for(i = 0; i < NumEntries; i ++)
    {
        sortindex = ip_array[i].sortindex;
        if(ip_array[sortindex].iplong != ip)
        {
            if(ip) statistics.ipnums ++;
            ip = ip_array[sortindex].iplong;
        }
    }
    return(statistics.ipnums);
}


void FindCachedEntries(unsigned long NumEntries)
{
    char    *dom;
    unsigned long   sortindex = 0, ip = -1;
    unsigned long   count = 0, i;

    for(i = 0; i < NumEntries; i ++)
    {
        sortindex = ip_array[i].sortindex;
        if((ip_array[sortindex].iplong != ip) && ip_array[sortindex].iplong)
        {
            ip  = ip_array[sortindex].iplong;
            dnscache[count].iplong = ip;
            dom = dnscache[count].domain;
            count ++;
        }
        ip_array[sortindex].dnsname = dom;
    }
}


void SortEntriesByIPNUM(unsigned long NumEntries) {
    int i, j, k, l, m;
    unsigned long li;
            
    unsigned long NoBottom = 0xFFFFFFFF;
    unsigned long NoTop    = 0;
    
    for(i = 0; i < NumEntries; i ++) 
    {
        li = (unsigned long) ip_array[ip_array[i].sortindex].iplong;
        if(li < NoBottom)
        {
            NoBottom = li;
            k = ip_array[i].sortindex;
            for(j = i; j != 0; j --) {
                l = ip_array[j-1].sortindex;
                ip_array[j].sortindex = l;
            }
            ip_array[0].sortindex = k;
            continue;
        }

        if(li >= NoTop) 
        {
            NoTop = li;
            continue;
        }

        for(j = 0; j < i; j++) {
            if(((unsigned long )ip_array[ip_array[j].sortindex].iplong) > li) 
            {
                m = ip_array[i].sortindex;
                for(k = i; k > j; k --) {
                    l = ip_array[k-1].sortindex;
                    ip_array[k].sortindex = l;
                }
                ip_array[j].sortindex = m;
                j = i;
            }
        }   
    }
}


/* DNSLookup - Performs the reverse DNS lookup on an IP address.        */
unsigned int
DNSLookup(struct dnsrec *dnsentry)
{
    struct hostent *hostdata;
    unsigned char   ipnum[4];

    ipnum[0] = (unsigned char )((dnsentry->iplong >> 24) & 0xFF), 
    ipnum[1] = (unsigned char )((dnsentry->iplong >> 16) & 0xFF),
    ipnum[2] = (unsigned char )((dnsentry->iplong >> 8)  & 0xFF),
    ipnum[3] = (unsigned char )(dnsentry->iplong & 0xFF);

    /* Look up domain name from IP address      */
    hostdata = gethostbyaddr((const char *) ipnum, 4, AF_INET);
    if(hostdata == NULL)
    {
        /* Domain name not found.  Mark as not found.   */
        sprintf(dnsentry->domain, "%d.%d.%d.%d",
                ipnum[0], ipnum[1], ipnum[2], ipnum[3]);
        return(0);
    }
    strcpy(dnsentry->domain, hostdata->h_name);
    return(1);
}


void PrintStats(FILE *output)
{
    fprintf(output, "\n");
    fprintf(output, "Logresolve (v3.0) statistics:\n\n");
    fprintf(output, "Already resolved in log file    :  %10ld\n", statistics.pre_resolved);
    fprintf(output, "Resolved from fake IP addresses :  %10ld\n", statistics.fake_ip_addr);
    fprintf(output, "Unique IP addresses:               %10ld\n", statistics.ipnums);
    fprintf(output, "Successful cache look-ups:         %10ld\n", statistics.hits);
    fprintf(output, "Resolved without a cache look-up:  %10ld\n", statistics.psuedo_hits);
    fprintf(output, "Cache evictions then DNS look-up:  %10ld\n", statistics.evictions);
    fprintf(output, "Successful DNS look-ups:           %10ld\n", statistics.resolved);
    fprintf(output, "Unsuccessful DNS look-ups:         %10ld\n", statistics.noname);
    fprintf(output, "===============================================\n");
    fprintf(output, "Total pre-resolved entries:        %10ld\n", statistics.pre_resolved + statistics.fake_ip_addr);
    fprintf(output, "Total cache hits:                  %10ld\n", statistics.hits + statistics.psuedo_hits);
    fprintf(output, "Total DNS look-ups:                %10ld\n", statistics.lookups + statistics.evictions);
    fprintf(output, "===============================================\n");
    fprintf(output, "Total log file entries:            %10ld\n\n", statistics.entries);
}


/*
 * gets a line from stdin
 */
int getline(FILE *in, char *s, int n)
{
    int     stcont = 1;
    while(stcont)
    {
        if(!fgets(s, n, in))
            return(1);
        DELETECRLF(s);
        if(!*s)
            continue;
        stcont = 0;
    }
    return(0);
}
